      program circorbit

c     -- fitting of a circular orbit to acceleration-P or Pdot-P data
c     -- version 2: monte carlo version 
c
c     Example:  circorbit w47.dat 1 2 5 -eya0.05 
c
c               will read columns 1, 2 and 5 of file w47.dat assuming an
c               absolute y-error of 0.05 m/s/s
c               probably necessary that first character in each line is
c               blank
c
c
c     -- direct all questions and bugs to >>>mkramer@jb.man.ac.uk<<<
c
c
c     february 2000 - michael kramer
c     october  2000 - mk
c
c     April 2003 - mk: fixed bug in conversion from pdot to acc


      implicit none

c     -- to handle input...
      integer maxdata, ncolmax
      parameter (maxdata=100)
      parameter (ncolmax=20)

      real*8 col(ncolmax)
      real*8 x(maxdata), y(maxdata)
      real*8 dx(maxdata), dy(maxdata), t(maxdata)
      character filename*30, line*120
      integer numcol, getcolnum, i, iargc, narg,xcol, ycol

c     -- these arrays will contain working numbers, e.g. squared,
c        scaled etc...
      real*8 x2(maxdata), y2(maxdata), dy2(maxdata)
      real*8 dyall, dxall
      integer tcol, n, ndata, errorsx, errorsy, dycol, dxcol
      logical accel, post

c     -- fitting stuff...
      integer ma, mamax
      parameter (mamax=6)
      real*8 a(mamax), u(maxdata,mamax), v(mamax,mamax), w(mamax)
      real*8 chisq
      character*80 title, numout

c--   -- fitting and scaling --
      real*8 xmin,xmax,x0, xscale ! x-scaling to avoid numerical problems
      real*8 yscale, ddx
      real*8 anew2, anew1, anew0

c     -- the result - now arrays for monte-carlo
      integer ntry, itry, iseed, secnds
      parameter (ntry=10000) 
      real*8 Pbi(ntry), P0i(ntry), asinici(ntry), k1i(ntry), k2i(ntry)
c     -- trial xtry's and ytry's have to be arrays to store used 
c        values
      real*8 xtry(maxdata), ytry(maxdata), gasdev, ran1
      real*8 phii(ntry,maxdata), t0i(ntry,maxdata),dd, arg1, arg2
      real*8 phi0
      logical distplot, phasedist, ellonly

c     -- final result --

      real*8 Pb, P0, asinic, k1, k2
      real*8 dPb, dpbl, dpbu, dP0, dp0l,dp0u
      real*8 dasinic, dasinicl, dasinicu, dk1, dk1l, dk1u
      real*8 dk2, dk2l, dk2u

      real*8 thresh, twopi,c 
      parameter (thresh=1.0d-12)
      parameter (twopi=6.283185)
      parameter (c=2.99792456e8)


      external fpoly
      
      data errorsx/0/
      data errorsy/0/
      data ma/3/
      data accel/.true./
      data post/.false./
      data distplot/.false./
      data phasedist/.false./
      data ellonly/.false./

      narg=iargc()
      if (narg.lt.4) then
         write(*,111) 
 111     format('USAGE: circorbit <file> <T-col> <P-col> ',
     +        '<Acc/Pdot-col> ',/,
     +        17x,'[-eyc<dy-col>] [-eyr<dy-val>] -[eya<dy-val>]',/,
     +        17x,'[-exc<dy-col>] [-exr<dy-val>] -[exa<dy-val>]',/,
     +        17x,'[-na] [-ps<title>] [-dist] [-phase] [-ellonly]')
         write(*,*) 'Epochs (T-col) are expected to be in JD/MJD!'
         write(*,*) 'Periods (P-col) are expected to be in ms!'
         write(*,*) 'Error estimates for Pdot/acc must be given:'
         write(*,*) '-eyc  -  specify dy column'
         write(*,*) '-eya  -  specify uniform absolute dy-value'
         write(*,*) '-eyr  -  specify uniform relative dy-value'
         write(*,*) 
         write(*,*) 'Error estimates for periods may be given:'
         write(*,*) '-exc  -  specify dx column'
         write(*,*) '-exa  -  specify uniform absolute dx-value'
         write(*,*) '-exr  -  specify uniform relative dx-value'
         write(*,*) 
         write(*,*) '-na    - flag for non-acceleration data'
         write(*,*) '-ps    - produce PS-file, may add string as title'
         write(*,*) '-dist  - plot MC-distributions for orbital parms'
         write(*,*) '-phase - plot MC-distributions for phases'
         write(*,*) '-ellonly - plot only ellipse (no iden string)' 
         write(*,*) 
         write(*,*) 'Additional output:  ''phases.dat''  containing:'
         write(*,*) '            Best binary period and...'
         write(*,*) '            epoch, phase, dphase, T0, dT0'
         write(*,*) 'Phase in orbital space rather than acc-space.'
         write(*,*) 
         write(*,*) 'Note: If the program crashes or the fit does not'
         write(*,*) '      converge, error estimates may be wrong.'
         write(*,*) 'Questions/bugs -> mkramer@jb.man.ac.uk'
         stop
      endif

      call getarg(1, filename)
      call getarg(2, line)
      read(line,*,err=900) tcol
      call getarg(3, line)
      read(line,*,err=900) xcol
      n=max(xcol,tcol)
      call getarg(4, line)
      read(line,*,err=900) ycol
      n=max(n,ycol)

      do i=5,narg
         call getarg(i, line)

c        -- decode y-error
         if (line(1:4).eq.'-eyc') then
            read(line(5:),*,err=900) dycol
            n=max(dycol,n)
            write(*,*) 'Reading y-errors from column...',dycol
            errorsy=1
         endif
         if (line(1:4).eq.'-eya') then
            read(line(5:),*,err=900) dyall
            write(*,*) 'Uniform absoulute y-error...',dyall
            errorsy=2
         endif
         if (line(1:4).eq.'-eyr') then
            read(line(5:),*,err=900) dyall
            write(*,*) 'Uniform relative y-error...', dyall
            errorsy=3
         endif

c        -- decode x-error
         if (line(1:4).eq.'-exc') then
            read(line(5:),*,err=900) dxcol
            n=max(dxcol,n)
            errorsx=1
         endif
         if (line(1:4).eq.'-exa') then
            read(line(5:),*,err=900) dxall
            errorsx=2
         endif
         if (line(1:4).eq.'-exr') then
            read(line(5:),*,err=900) dxall
            errorsx=3
         endif

         if (line(1:2).eq.'-na') then
            accel=.false.
            write(*,*) 'Reading non-acceleration data (Pdots).'
         endif

         if (line(1:3).eq.'-ps') then
            post=.true.
            title=line(4:)
         endif

         if (line(1:5).eq.'-dist') distplot=.true.
         if (line(1:7).eq.'-phases') phasedist=.true.
         if (line(1:8).eq.'-ellonly') ellonly=.true.

      enddo

c     -- read in data --
      open(78,file=filename,form='formatted',status='old',err=900)

      read(78,100) line
 100  format(a120)
      numcol=getcolnum(line)
      write(*,*) 'File contains ',numcol,' columns. '
      if (numcol.gt.ncolmax) then
         write(*,*) 'Number of input columns too large.'
         stop
      endif
      if (n.gt.numcol) stop 'Requested column does not exist.'

      n=0
      rewind(78)
 200  continue
         read(78,100,end=300) line
c         write(*,*) line
         read(line,*,err=950) (col(i), i=1, numcol)
         n=n+1
         if (n.gt.maxdata) then
            write(*,*) 'Maximum number of data reached: ',maxdata
            stop
         endif
         t(n)=col(tcol)
         x(n)=col(xcol)/1000.0 ! assuming ms -> seconds!
         y(n)=col(ycol)

c       -- get y-errors
         if (errorsy.eq.1) then
            dy(n)=col(dycol)
         else 
            if (errorsy.eq.2) then
               dy(n)=dyall
            else
               if (errorsy.eq.3) then
                  dy(n)=abs(dyall*y(i))
               else
                  stop 'No y-error estimate.'
               endif
            endif
         endif

c       -- get x-errors
         if (errorsx.eq.1) then
            dx(n)=col(dxcol)
         else 
            if (errorsx.eq.2) then
               dx(n)=dxall
            else
               if (errorsx.eq.3) then
                  dx(n)=abs(dxall*x(i))
               else
                  dx(n)=0.0
               endif
            endif
         endif


         goto 200
 300  continue
      ndata=n


c     -- prepare x-scaling 
      call getlims(x,ndata,xmin,xmax,.false.)
      x0=0.5*(xmin+xmax)  ! new zero point
      yscale=x0/c       ! factor for converting pdot's to a's
      ddx=abs(xmax-xmin) ! that is the present scale
      xscale=100.0/ddx   ! bring it to about 100

      if (.not.accel) then
         do i=1, ndata
            y(i)=y(i)/yscale    ! calculate accelerations
            dy(i)=dy(i)/yscale
         enddo
      endif

c     iseed=-643746 ! for those f77-libraries without secnds
      iseed=-int(secnds(0.0))
      ddx=dble(ran1(iseed))  ! just some dummy to initialize

      do itry=1, ntry  ! monte carlo loop

         do i=1, ndata
c        - use x-value varied with given uncertainty
            xtry(i)=dx(i)*dble(gasdev(iseed))+x(i)
            x2(i)=xscale*(xtry(i)-x0)
c          - use y-value varied with given uncertainty
            ytry(i)=dy(i)*dble(gasdev(iseed))+y(i)
c     --    square trial value...
            y2(i)=ytry(i)**2
            dy2(i)=1.0          ! uniform weight in monte carlo fit...
         enddo

c     -- fit the values to parabola (uniform weight)
         call svdfit(x2,y2,dy2,ndata,
     +               a,ma,u,v,w,maxdata,mamax,chisq,fpoly)

c     -- correct for x-scaling
         anew2=a(3)*xscale*xscale
         anew1=xscale*a(2)-2.0*anew2*x0
         anew0=a(1)-x0*(anew1+anew2*X0)

c     -- all values are now accelerations -- 
         P0i(itry)=-0.5*anew1/anew2
         if (.not.accel) then   ! originally pdot data
                                ! rescale now with proper P0
            yscale=(P0i(itry)/x0)**2
            anew0=anew0*yscale
            anew1=anew1*yscale
            anew2=anew2*yscale
         endif

C     -- calculate parameters --
         Pbi(itry)=twopi/sqrt(-anew2)*c/P0i(itry)
c     - k2 is y-axis of ellipse -
         k2i(itry)=sqrt(anew0+anew1*P0i(itry)+anew2*P0i(itry)*P0i(itry))
c     -- k1 is x-axis of ellipse --
         k1i(itry)=k2i(itry)/sqrt(-anew2)
         asinici(itry)=Pbi(itry)/twopi/p0i(itry)
     +        *sqrt(p0i(itry)**2-anew0/anew2)

C     -- calculate phi for each try variation 
c        problem: since our phi will be defined on -0.5 to 0.5
c        the distribution could be cut in half, if our phi is close
c        to abs(0.5), by pushing the randomly varied phi over the
c        phase wrap, i.e. creating two halves at -0.5 and +0.5
c        Fix: get an idea where phi is ending up and then look at the
c        difference. Unless the error in Phi is huge we can calculate
c        the difference. If it is larger than 0.75, say, in magnitude
c        then there was a phase wrap which we can correct.

         do i=1, ndata

c          -- trial phi
            dd=xtry(i)-p0i(itry)
            arg1=k1i(itry)*ytry(i)
            arg2=k2i(itry)*dd         
            if (abs(arg2).gt.thresh) then ! check numerical problems for
                                          ! 90 deg
               phii(itry,i)=atan2(arg1,arg2)/twopi ! between -0.5 and 0.5
            else
               phii(itry,i)=0.25
               if (arg1.lt.0.0) phii(itry,i)=-phii(itry,i)
            endif
            phii(itry,i)=-phii(itry,i) ! phi was in acc-P space, now in orbit

c         - expected phi:
            dd=x(i)-p0i(itry)
            arg1=k1i(itry)*y(i)
            arg2=k2i(itry)*dd         
            if (abs(arg2).gt.thresh) then 
               phi0=atan2(arg1,arg2)/twopi ! between -0.5 and 0.5
            else
               phi0=0.25
               if (arg1.lt.0.0) phi0=-phi0
            endif
            phi0=-phi0
            dd=phi0-phii(itry,i)
            if (abs(dd).gt.0.75) then ! phase wrap..
               if (dd.gt.0.0) then ! ...from + to -
                  phii(itry,i)=phii(itry,i)+1.0
               else                ! ...from - to +
                  phii(itry,i)=phii(itry,i)-1.0
               endif
            endif

c           -- calculate T0 from that...(nb. pbi is in sec!)
            t0i(itry,i)=t(i)-phii(itry,i)*Pbi(itry)/86400.0            
         enddo

c         write(*,*) anew2, anew1, anew0

      enddo ! end of MC-loop

      if (distplot) then
         if (post) then
            call pgbegin(0,'dist.ps/ps',3,2)
         else
            call pgbegin(0,'/xw',3,2)
         endif
      endif
         
      call checkdist(P0i,ntry,p0,dp0,dp0l,dp0u,1000.0,distplot)
      call checkdist(Pbi,ntry,pb,dpb,dpbl,dpbu,1.0/86400.,distplot)
      call checkdist(asinici,ntry,asinic,dasinic,
     +     dasinicl,dasinicu,1.0,distplot)
      call checkdist(k1i,ntry,k1,dk1,dk1l,dk1u,1.0,distplot)
      call checkdist(k2i,ntry,k2,dk2,dk2l,dk2u,1.0,distplot)
      if (distplot) call pgend

      call writePhase(phii,t0i,ntry,ndata,ntry,maxdata,t,Pb,dPb,
     +                phasedist)

      pb=pb/86400.0d0   ! convert to days...
      dpb=dpb/86400.0d0
      dpbu=dpbu/86400.0d0
      dpbl=dpbl/86400.0d0
      call printnum(P0,dP0,numout,i)
      write(*,*) 'Pulse period [s]: ',numout(1:i)
      write(*,*) ' lower/upper err: ',dp0l,dp0u
      call printnum(Pb,dPb,numout,i)
      write(*,*) 'Orb-Period [d]:   ',numout(1:i)
      write(*,*) ' lower/upper err: ',dpbl,dpbu
      call printnum(asinic,dasinic,numout,i)
      write(*,*) 'X=asini*c [s]:    ',numout(1:i)
      write(*,*) ' lower/upper err: ',dasinicl,dasinicu
      write(*,*)
      call printnum(k1,dk1,numout,i)
      write(*,*) 'x-Axis of ellipse k1: ',numout(1:i)
      call printnum(k2,dk2,numout,i)
      write(*,*) 'y-Axis of ellipse k2: ',numout(1:i)

      call plotall(x,y,dy,ndata,P0,k1,k2,anew0,anew1,anew2,
     +     pb,asinic,dp0,dpb,dasinic,post,title,filename,ellonly)

    
      stop
 900  continue
      write(*,*) 'Could not parse arguments.'
      stop

 950  continue
      write(*,*) 'Could not read line. Skipping: '
      write(*,*) line
      goto 200

      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine plotall(x,y,dy,ndata,x0,k1,k2,
     +     anew0,anew1,anew2,
     +     pb,asinic,dx0,dpb,dasinic,
     +     post,title,filename,ellonly)
      
      implicit none

      integer ndata, maxdata
      parameter (maxdata=1000)
      real*8 x(ndata),y(ndata),dy(ndata),y2(maxdata)
      real*8 anew0, anew1, anew2, k1, k2, x0, pb, asinic
      real*8 dx0,dpb,dasinic
      real*8 dx, xmin, xmax, ymin, ymax
      logical ellonly

c     -- to draw the result...
      integer nfit, i
      parameter (nfit=100)
      real*8 xfit(nfit), yfit(nfit)
      character title*(*), filename*(*), line*120
      real*8 twopi
      parameter (twopi=6.283185)

      logical margin, nomargin, post
      
      data margin/.true./
      data nomargin/.false./

c     -- plot data in an acc-P diagram
      if (.not.post) then
         call pgbegin(0,'/xw',1,1)
         title=' '
      else
         call pgbegin(0,'/ps',1,1)
         if (.not.ellonly) call pgiden ! useful for records
      endif

      do i=1, ndata
         x(i)=x(i)*1000.0d0 ! convert to ms
         y2(i)=y(i)*y(i)
      enddo
      x0=x0*1000.0d0
      dx0=dx0*1000.0d0
      k1=k1*1000.0d0
      
      xmin=x0-1.25*k1
      xmax=x0+1.25*k1
      ymin=-1.25*k2
      ymax=1.25*k2

      if (ellonly) then
         call pgvport(0.1,0.95,0.1,0.95)
      else
         call pgvport(0.1,0.45,0.1,0.95)
      endif
      call frame(xmin,xmax,ymin,ymax)

c     -- plot derived ellipse
      dx=twopi/float(nfit-2)
      do i=0, nfit-1
         xfit(i+1)=k1*cos(dx*float(i))+x0
         yfit(i+1)=k2*sin(dx*float(i))
      enddo
      call pgsls(2)
      call pgline8(nfit,xfit,yfit)

c     -- plot data points
      call pgsls(1)
      if (.not.post) call pgsci(3)
c      call pgpt8(ndata,x,y,2)
      call errorbars(ndata,x,y,dy)
      call pgsci(1)
      call pgsls(1)

      call pglabel('Period (ms)','Acceleration (m/s/s)',title)

      if (.not.ellonly) then !  set up parabola plot --
         call getlims(x,ndata,xmin,xmax,margin)
         call getlims(y2,ndata,ymin,ymax,margin)
         call pgvport(0.55,0.95,0.4,0.95)
         ymin=0.0d0             ! plot from zero acc
         call frame(xmin,xmax,ymin,ymax)

c        -- plot fitted parabola using same x-values
         dx=(xmax-xmin)/float(nfit)
         do i=1, nfit
            xfit(i)=xmin+float(i)*dx
            yfit(i)=anew0+0.001*xfit(i)*(anew1+anew2*xfit(i)*0.001)
         enddo
         call pgsls(2)
         call pgline8(nfit,xfit,yfit)

c        -- plot values --
         call pgsls(1)
         if (.not.post) call pgsci(3)
         call pgsfs(1)
         call pgpt8(ndata,x,y2,-31)
c        call errorbars(ndata,x,y2,dy2)
         call pgsci(1)

          call pglabel('Period (ms)','Square of Acceleration',' ')

c     -- print values --
          call pgvport(0.6,0.95,0.05,0.26)
          call pgwindow(0.0,1.0,0.0,1.0)
          write(line,*) 'File: ',filename
          call pgtext(0.0,1.0,'File: '//filename(1:30))
          call printnum(x0,dx0,line,i)
          call pgtext(0.0,0.8,'P (ms): '//line(1:i))
          call printnum(Pb,dPb,line,i)
          call pgtext(0.0,0.6,'Pb (d): ' //line(1:i))
          call printnum(asinic,dasinic,line,i)
          call pgtext(0.0,0.4,'x (s): '//line(1:i))
       endif

      call pgend

      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine writePhase(phii,t0i,ntry,ndata,maxtry,maxdata,t,
     +                      Pb,dPb,distplot)

      integer ntry,ndata,maxtry,maxdata

      real*8 phii(maxtry,maxdata), t0i(maxtry,maxdata)
      real*8 t(ndata), pb, dpb, d1, d2

      integer maxtryLOCAL
      parameter (maxtryLOCAL=10000)

      integer i, itry
      real*8 x(maxtryLocal)
      real*8 phi, dphi, t0, dt0

      logical distplot

      IF (maxtryLocal.lt.maxtry) then
         write(*,*) 'WRITEPHASE: local maxtry < global maxtry'
         stop
      endif


      write(*,*) 'Writing phases...'
      open(99,file='phases.dat',form='formatted',status='unknown')

      if (distplot) call pgbegin(0,'/xw',6,4)

      write(99,100) pb, dpb
 100  format('Pb  ',f15.6,2x,f15.6)
      do i=1, ndata         
         do itry=1, ntry
            x(itry)=phii(itry,i)
         enddo
         call checkdist(x,ntry,phi,dphi,d1,d2,1.0,distplot)
c        and check for phase wrap again
         if (phi.gt.0.5d0)  phi=phi-1.0d0
         if (phi.lt.-0.5d0) phi=phi+1.0d0
         do itry=1, ntry
            x(itry)=t0i(itry,i)
         enddo
         call checkdist(x,ntry,t0,dt0,d1,d2,1.0,.false.)

         write(99,200) t(i), phi, dphi, t0, dt0
 200     format(1x,f11.5,2x,f8.5,1x,f8.5,2x,f11.5,1x,f11.5)
      enddo

      close(99)

      if (distplot) call pgend
      write(*,*) 'Writing phases...done.'


      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc











